"""
mciutil.cli.mideu

provides functionality for cli tool mideu
"""
from __future__ import print_function

import os.path
import logging
import argparse

from mciutil import _version
from mciutil.cli.common import add_logging_arg_group, add_source_format_arg
from mciutil.cli.extract import extract_command
from mciutil.cli.convert import convert_command

LOGGER = logging.getLogger(__name__)


def cli_entry():
    """
    mideu main cli entry

    :return: None
    """
    args = _get_cli_parser().parse_args()
    _main(args)


def _main(args):
    """
    mideu main processing

    :param args: argparse arguments
    :return: exit code
    """
    if not hasattr(args, "loglevel"):
        print(__name__ + " ("+_version.get_versions()['version']+")")
        print("try --help for information")
        return

    logging.basicConfig(
        level=args.loglevel,
        format="%(asctime)s:%(name)s:%(lineno)s:%(levelname)s:%(message)s"
    )

    # exit if input file does not exist
    if not os.path.isfile(args.input):
        print("Input file not found - {0}".format(args.input))
        exit(8)

    # do command level processing
    args.func(args)

    print("Done!")


def _get_cli_parser():
    """
    mideu argparse parser create

    :return: parser
    """
    parser = argparse.ArgumentParser(
        description="MasterCard IPM file formatter ({version})".format(
            version=_version.get_versions()['version'])
    )
    parser.add_argument("--version", action="version",
                        version="%(prog)s ("+_version.get_versions()['version']+")",
                        help="Get version information")

    subparsers = parser.add_subparsers(help="Sub-command help")

    # Extract command
    extract_parser = subparsers.add_parser("extract", help="Extract help")
    extract_parser.set_defaults(func=extract_command)
    _add_common_args(extract_parser)
    _add_extract_args(extract_parser)

    # Convert command
    convert_parser = subparsers.add_parser("convert", help="Convert help")
    convert_parser.set_defaults(func=convert_command)
    _add_common_args(convert_parser)

    return parser


def _add_common_args(parser):
    """
    mideu add common cli arguments

    :param parser: the argparse parser
    :return: None
    """
    parser.add_argument("input", help="Input IPM file name")
    add_source_format_arg(parser)
    add_logging_arg_group(parser)


def _add_extract_args(parser):
    """
    mideu add extract subcommand arguments

    :param parser: the argparse parser
    :return: None
    """
    mongo_arg_group = parser.add_argument_group("mongo output options")
    mongo_arg_group.add_argument("--mongo",
                                 help="add to mongo",
                                 action="store_true")
    mongo_arg_group.add_argument("--mongohost", help="mongo hostname")
    mongo_arg_group.add_argument("--mongodb", help="mongo db")

    csv_arg_group = parser.add_argument_group("csv output options")
    csv_arg_group.add_argument("--csvoutputfile", help="Output filename")


if __name__ == "__main__":
    _main(_get_cli_parser().parse_args())



"""
mciutil function library

Contains functions to work with MasterCard formatted files
"""
from __future__ import print_function

import logging
import sys
import struct
import datetime
import decimal
import codecs
import binascii

import bitarray
import hexdump

LOGGER = logging.getLogger(__name__)


def unblock(blocked_data):
    """
    Unblocks a 1014 byte blocked string

    :param blocked_data: String containing 1014 blocked data or file
    :return: list of records in file
    """

    file_pointer = 0
    unblock_data = []

    while file_pointer <= len(blocked_data):
        unblock_data.append(blocked_data[file_pointer:file_pointer+1012])
        file_pointer += 1014

    return vbs_unpack(b("").join(unblock_data))


def vbs_unpack(vbs_data):
    """
    Unpacks a variable blocked string into a list of records

    :param vbs_data: string containing repeating binary length(4) + data
        records in a single string.
    :return: list of unpacked records
    """

    vbs_pointer = 0     # position in file
    line_data = []      # output string

    while vbs_pointer < len(vbs_data):

        # get record length
        record_length_raw = vbs_data[vbs_pointer:vbs_pointer+4]

        # increment file pointer
        vbs_pointer += 4

        # get numerical record length
        record_length = struct.unpack(">i", record_length_raw)[0]
        LOGGER.debug("record_length=%s", record_length)

        # exit if last record (length=0)
        if record_length == 0:
            break

        # get record data
        record = vbs_data[vbs_pointer:vbs_pointer+record_length]

        # add record to output
        line_data.append(record)

        # increment file pointer
        vbs_pointer += record_length

    return line_data


def block(unblocked_data):
    """
    Blocks a list of records to 1014 byte blocked VBS records

    :param unblocked_data: list containing records
    :return: string containing repeating binary length(4) + data records in a
        single string including 1014 block markers
    """

    unblocked_data = vbs_pack(unblocked_data)

    pad_char = b("\x40")
    blocked_data = []
    block_count = 0

    while block_count <= len(unblocked_data):
        blocked_data.append(unblocked_data[block_count:block_count+1012])
        blocked_data.append(pad_char*2)
        block_count += 1012

    # add the pad characters
    pad_count = block_count - len(unblocked_data)
    print(str(pad_count) + " pad characters added")
    blocked_data.append(pad_char * pad_count)

    # return blocked_data
    return b("").join(blocked_data)


def vbs_pack(records):
    """
    Packs a list of records into a variable blocked string

    :param records: list containing records to be packed
    :return: string containing repeating binary length(4) + data
        records in a single string
    """

    vbs_data = []

    for record in records:
        # get the length of the record
        record_length = len(record)
        # convert length to binary
        record_length_raw = struct.pack(">i", record_length)
        # add length to output data
        vbs_data.append(record_length_raw)
        # add data to output
        vbs_data.append(record)

    # add zero length to end of record
    vbs_data.append(struct.pack(">i", 0))

    return b("").join(vbs_data)


def flip_message_encoding(message, bit_config, source_format):
    """
    Flip the encoding of an ISO8583 style file between ASCII and EBCDIC

    :param message: The data to be flipped
    :param bit_config: dictionary of bit mapping configuration
    :param source_format: The encoding of the source {ebcdic, ascii}
    :return: message encoded
    """
    message_length = len(message)-20
    (message_type_indicator, binary_bitmap, message_data) \
        = struct.unpack("4s16s" + str(message_length) + "s", message)

    flipped_message = b("")

    # add the message type
    if source_format == 'ebcdic':
        flipped_message += _convert_text_eb2asc(message_type_indicator)
    else:
        flipped_message += _convert_text_asc2eb(message_type_indicator)

    message_pointer = 0
    bitmap_list = _get_bitmap_list(binary_bitmap)

    # add back the bitmap - no encoding
    flipped_message += binary_bitmap

    for bit in range(2, 128):   # cycle each bit
        if bitmap_list[bit]:    # if bit is on for message
            if bit in bit_config:   # if config available for bit
                return_message, message_increment = \
                    _flip_element_encoding(
                        bit_config[bit],
                        message_data[message_pointer:],
                        source_format)
            else:
                print("No config found for bit {0}".format(bit))
                raise Exception("Config missing for bit {0}".format(bit))

            # Increment the message pointer and process next field
            message_pointer += message_increment
            flipped_message += return_message
    return flipped_message


def _flip_element_encoding(bit_config, message_data, source_format):
    """
    Converts a field in an iso8583 style message from ascii to ebcdic

    :param bit_config: dict of field iso8583 bits mapping
    :param message_data: the message containing the field
    :param source_format: encoding of source -- ebcdic or ascii
    :returns: converted string
    :returns: pointer to next field in message

    """

    flipped_element = b("")

    field_length = bit_config['field_length']
    print("processing field {0}".format(bit_config["field_name"]))

    length_size = _get_field_length(bit_config)

    if length_size > 0:
        field_length_string = message_data[:length_size]
        print("field_length_string {0}, {1}".format(length_size, field_length_string))
        if source_format == 'ebcdic':
            field_length_string = _convert_text_eb2asc(field_length_string)
            field_length = int(field_length_string)
        else:
            field_length = int(field_length_string)
            field_length_string = _convert_text_asc2eb(field_length_string)

        flipped_element += field_length_string

    field_data = message_data[length_size:length_size+field_length]

    field_processor = _set_parameter(bit_config,
                                     'field_processor')
    # flip except for ICC field
    if field_processor != 'ICC':
        if source_format == 'ebcdic':
            converted_data = _convert_text_eb2asc(field_data)
        else:
            converted_data = _convert_text_asc2eb(field_data)
        if len(field_data) != len(converted_data):
            raise Exception(
                "Conversion returned different lengths\n{0}\n{1}".format(
                    hexdump.hexdump(field_data, result="return"),
                    hexdump.hexdump(converted_data, result='return')
                )
            )
        flipped_element += converted_data
    else:  # Add ICC data as is
        flipped_element += field_data

    return flipped_element, field_length + length_size


def get_message_elements(message, bit_config, source_format):
    """
    Convert ISO8583 style message to dictionary

    :param message: The message in ISO8583 based format

    * Message Type indicator - 4 bytes
    * Binary bitmap - 16 bytes (Reads DE1 and DE2)
    * Message data - Remainder of record

    :param bit_config: dictionary of bit mapping configuration
    :param source_format: string indicating encoding of data (ascii|ebcdic)
    :return: dictionary of message elements

    * key = 'MTI' message type indicator
    * key = 'DEx' data elements
    * key = 'PDSxxxx' private data fields
    * key = 'TAGxxxx' icc fields

    """
    LOGGER.debug("processing record %s", hexdump.hexdump(message))
    # split raw message into components MessageType(4B), Bitmap(16B),
    # Message(l=*)
    message_length = len(message)-20
    (message_type_indicator, binary_bitmap, message_data) \
        = struct.unpack("4s16s" + str(message_length) + "s", message)

    return_values = dict()

    # add the message type
    if source_format == 'ebcdic':
        return_values["MTI"] = _convert_text_eb2asc(message_type_indicator)
    else:
        return_values["MTI"] = message_type_indicator

    message_pointer = 0
    bitmap_list = _get_bitmap_list(binary_bitmap)

    for bit in range(2, 128):
        if bitmap_list[bit]:
            LOGGER.debug("processing bit %s", bit)
            return_message, message_increment = \
                _process_element(bit,
                                 bit_config[bit],
                                 message_data[message_pointer:],
                                 source_format)

            # Increment the message pointer and process next field
            message_pointer += message_increment
            return_values.update(return_message)

    return return_values


def _process_element(bit, bit_config, message_data, source_format):
    """
    Processes a message bit element

    :param bit: DE bit
    :param bit_config: message bit configuration
    :param message_data: the data to be processed
    :param source_format: EBCDIC or ASCII
    :returns:
        dictionary: field values
        message incrementer: position of next message
    """

    field_length = bit_config['field_length']

    length_size = _get_field_length(bit_config)

    if length_size > 0:
        field_length_string = message_data[:length_size]
        if source_format == 'ebcdic':
            field_length_string = _convert_text_eb2asc(field_length_string)
        field_length = int(field_length_string)

    field_data = message_data[length_size:length_size+field_length]

    field_processor = _set_parameter(bit_config,
                                     'field_processor')

    # do ascii conversion except for ICC field
    if field_processor != 'ICC':
        if source_format == 'ebcdic':
            field_data = _convert_text_eb2asc(field_data)

    # if field is PAN type, mask the card value
    if field_processor == 'PAN':
        field_data = _mask_pan(field_data)

    # do field conversion to native python type
    field_data = _convert_to_type(field_data, bit_config)

    return_values = dict()

    # add value to return dictionary
    return_values["DE" + str(bit)] = field_data

    # if a PDS field, break it down again and add to results
    if field_processor == 'PDS':
        return_values.update(_get_pds_fields(field_data))

    # if a DE43 field, break in down again and add to results
    if field_processor == 'DE43':
        return_values.update(_get_de43_fields(field_data))

    # if ICC field, break into tags
    if field_processor == 'ICC':
        return_values.update(_get_icc_fields(field_data))

    return return_values, field_length + length_size


def _set_parameter(config, parameter):
    """
    Checks for parameter value and sets if present

    :param config: bit configuration list
    :param parameter: the configuration item to set
    :return: string with value of parameter
    """
    if parameter in config:
        return config[parameter]
    else:
        return ""


def _mask_pan(field_data):
    """
    Mask a pan number string

    :param field_data: unmasked pan
    :return: masked pan
    """
    # if field is PAN type, mask the card value
    return field_data[:6] \
        + (b("*") * (len(field_data)-9)) \
        + field_data[len(field_data)-3:len(field_data)]


def _convert_to_type(field_data, bit_config):
    """
    Field conversion to native python type

    :param field_data: Data to be converted
    :param bit_config: Configuration for bit
    :return: data in required type
    """
    field_python_type = _set_parameter(bit_config, 'python_field_type')

    if field_python_type == "int":
        field_data = int(field_data)
    if field_python_type == "decimal":
        field_data = decimal.Decimal(field_data)
    if field_python_type == "long":
        field_data = int(field_data)
    if field_python_type == "datetime":
        field_data = datetime.datetime.strptime(
            field_data, "%y%m%d%H%M%S")
    return field_data


def _get_field_length(bit_config):
    """
    Determine length of iso8583 style field

    :param bit_config: dictionary of bit config data
    :return: length of field
    """
    length_size = 0

    if bit_config['field_type'] == "LLVAR":
        length_size = 2
    elif bit_config['field_type'] == "LLLVAR":
        length_size = 3

    return length_size


def _convert_text_eb2asc(value_to_convert):
    """
    Converts a string from ebcdic to ascii

    :param value_to_convert: The ebcdic value to convert
    :return: converted ascii text
    """

    val = codecs.encode(codecs.decode(value_to_convert, "cp500"), "latin-1")
    return val


def _convert_text_asc2eb(value_to_convert):
    """
    Converts a string from ebcdic to ascii

    :param value_to_convert: The ascii value to convert
    :return: converted ebcdic text
    """

    return codecs.encode(codecs.decode(value_to_convert, "latin-1"), "cp500")


def _get_bitmap_list(binary_bitmap):
    """
    Get list of bits from binary bitmap

    :param binary_bitmap: the binary bitmap to be returned
    :return: the list containing bit values. Bit 0 contains original binary
             bitmap
    """

    working_bitmap_list = bitarray.bitarray(endian='big')
    working_bitmap_list.frombytes(binary_bitmap)

    # Add bit 0 -> original binary bitmap
    bitmap_list = [binary_bitmap]

    # add bits from bitmap
    bitmap_list.extend(working_bitmap_list.tolist())

    return bitmap_list


def _get_pds_fields(field_data):
    """
    Get MasterCard pds fields from iso field

    :param field_data: the field containing pds fields
    :return: dictionary of pds key values
             key in the form PDSxxxx where x is zero filled number of pds
    """

    field_pointer = 0
    return_values = {}

    while field_pointer < len(field_data):
        # get the pds tag id
        pds_field_tag = field_data[field_pointer:field_pointer+4]
        LOGGER.debug("pds_field_tag=[%s]", pds_field_tag)

        # get the pds length
        pds_field_length = int(field_data[field_pointer+4:field_pointer+7])
        LOGGER.debug("pds_field_length=[%i]", pds_field_length)

        # get the pds data
        pds_field_data = \
            field_data[field_pointer+7:field_pointer+7+pds_field_length]
        LOGGER.debug("pds_field_data=[%s]", str(pds_field_data))
        return_values["PDS" + pds_field_tag.decode()] = pds_field_data

        # increment the fieldPointer
        field_pointer += 7+pds_field_length

    return return_values


def _get_icc_fields(field_data):
    """
    Get de55 fields from message

    :param field_data: the field containing de55
    :return: dictionary of de55 key values
             key is tag+tagid
    """
    TWO_BYTE_TAG_PREFIXES = [b("\x9f"), b("\x5f")]

    field_pointer = 0
    return_values = {"ICC_DATA": binascii.b2a_hex(field_data)}

    while field_pointer < len(field_data):
        # get the tag id (one byte)
        field_tag = field_data[field_pointer:field_pointer+1]
        # set to 2 bytes if 2 byte tag
        if field_tag in TWO_BYTE_TAG_PREFIXES:
            field_tag = field_data[field_pointer:field_pointer+2]
            field_pointer += 2
        else:
            field_pointer += 1

        field_tag_display = binascii.b2a_hex(field_tag)
        print("field_tag_display={0}".format(field_tag_display))
        field_length_raw = field_data[field_pointer:field_pointer+1]
        field_length = struct.unpack(">B", field_length_raw)[0]

        LOGGER.debug("%s", format(field_tag_display))
        LOGGER.debug(field_length)

        # get the pds data
        de_field_data = field_data[field_pointer+1:field_pointer+field_length+1]
        de_field_data_display = binascii.b2a_hex(de_field_data)
        LOGGER.debug("%s", de_field_data_display)
        return_values["TAG" + field_tag_display.upper().decode()] = de_field_data_display

        # increment the fieldPointer
        field_pointer += 1+field_length

    return return_values


def _get_de43_fields(de43_field):
    """
    get pds 43 field breakdown
    :param de43_field: data of pds 43
    :return: dictionary of pds 43 sub elements
    """

    de43_elements = {}
    de43_split = de43_field.split(b('\\'))
    de43_elements["DE43_NAME"] = de43_split[0].rstrip()
    de43_elements["DE43_ADDRESS"] = de43_split[1].rstrip()
    de43_elements["DE43_SUBURB"] = de43_split[2].rstrip()
    de43_elements["DE43_POSTCODE"] = de43_split[3][:4]
    de43_elements["DE43_STATE"] = de43_split[3][len(de43_split[3])-6:len(de43_split[3])-3]
    de43_elements["DE43_COUNTRY"] = de43_split[3][len(de43_split[3])-3:len(de43_split[3])]
    return de43_elements

if sys.version_info < (3,):
    def b(string):
        """
        Create a byte string field - Python 2.x

        :param string: input string
        :return: a byte array containing the string
        """
        return string
else:
    def b(string):
        """
        Create a byte string field - Python 3.x

        :param string: input string
        :return: a byte array containing the string
        """
        return codecs.latin_1_encode(string)[0]



#!/usr/bin/env python
from __future__ import absolute_import
import pymongo
from datetime import datetime


def insert(db):
    result = db.restaurants.insert_one(
        {
            "address": {
                "street": "2 Avenue",
                "zipcode": "10075",
                "building": "1480",
                "coord": [-73.9557413, 40.7720266]
            },
            "borough": "Manhattan",
            "cuisine": "Italian",
            "grades": [
                {
                    "date": datetime.strptime("2014-10-01", "%Y-%m-%d"),
                    "grade": "A",
                    "score": 11
                },
                {
                    "date": datetime.strptime("2014-01-16", "%Y-%m-%d"),
                    "grade": "B",
                    "score": 17
                }
            ],
            "name": "Vella",
            "restaurant_id": "41704620"
        }
    )

    print(result)
    print(result.inserted_id)


def find_data(db):
    # cursor = db.restaurants.find({"cuisine": "Australian"})
    # cursor = db.restaurants.find({"grades.grade": "A"})
    cursor = db.restaurants.find({"grades.score": {"$gt": 60},
                                  "cuisine": "Italian"}
                                 ).sort("address.zipcode", pymongo.ASCENDING)

    for document in cursor:
        print(document)

    print(
        db.restaurants.count({"grades.score": {"$gt": 60},
                              "cuisine": "Italian"})
    )


def format_string():
    item_count = 100000
    total_items = 200000
    print(
        "processed {percent_complete}%".format(
            percent_complete=(
                round(
                    item_count / float(total_items) * 100, 0
                )
            )
        )
    )


def mci_single_query(db):
    # print number of records
    print(db.mastercardtransactions.count({"DE43_POSTCODE": "4103"}))

    cursor = db.mastercardtransactions.find({"DE43_POSTCODE": "4103"})
    for doc in cursor:
        print(doc)


def mci_breakdown_by_mcc_by_ird(db):
    cursor = db.mastercardtransactions.aggregate(
        [
            # {"$match": {"DE43_POSTCODE": "4103"}},
            {"$sort": {"DE26": 1}},
            {"$group":
                {
                    "_id": {"mcc": "$DE26", "postcode": "$DE43_POSTCODE",
                            "ird": "$PDS0158"},
                    "count": {"$sum": 1},
                    "total": {"$sum": "$DE4"}
                }
             },
            {"$group":
                {
                    "_id": {"mcc": "$_id.mcc", "postcode": "$_id.postcode"},
                    "data": {"$push": "$$ROOT"}
                }

             },
            {"$group":
                {
                    "_id": "$_id.mcc",
                    "data": {"$push": "$$ROOT"}
                }
             }
        ]
    )
    for doc in cursor:
        print(doc)


def mci_queries(db):

    cursor = db.mastercardtransactions.aggregate(
        [
            {"$match": {"DE43_POSTCODE": "4103"}},
            {"$sort": {"DE42": 1}},
            {"$group":
                {
                    "_id": {"caid": "$DE42", "ird": "$PDS0158"},
                    "count": {"$sum": 1},
                    "total": {"$sum": "$DE4"}
                }
             },
            # groups into caid
            {"$group":
                {
                    "_id": "$_id.caid",
                    "data": {"$push": "$$ROOT"}
                }
             }

            # dictionary with caid and list of all transactions
            # {"$group":
            #     {
            #         "_id": "$DE42",
            #         "transactions": {"$push": "$$ROOT"}
            #     }
            # },
        ]
    )
    for doc in cursor:
        print(doc)


if __name__ == '__main__':
    client = pymongo.MongoClient("mongodb://192.168.99.100:27017")
    db = client['test']
    # insert(db)
    # find_data(db)
    # mci_queries(db)
    # mci_single_query(db)
    mci_breakdown_by_mcc_by_ird(db)
    # format_string()



"""
mciutil.cli.paramconv

provides functionality for cli tool paramconv
"""
from __future__ import print_function

import os.path
import argparse
import logging

from hexdump import hexdump

from mciutil import block, unblock, _version
from ..mciutil import _convert_text_eb2asc, _convert_text_asc2eb
from mciutil.cli.common import add_logging_arg_group, add_source_format_arg


def cli_entry():
    """
    paramconv main cli entry

    :return: None
    """
    args = _get_cli_parser().parse_args()
    _main(args)


def _get_cli_parser():
    """
    paramconv argparse parser create

    :return: parser
    """
    parser = argparse.ArgumentParser(
        description="MasterCard parameter file conversion utility ({version})".format(
            version=_version.get_versions()['version'])
    )
    parser.add_argument("input", help="MasterCard parameter file name")
    parser.add_argument("-o", "--output", help="Converted parameter file name")
    parser.add_argument("--version", action="version",
                        version="%(prog)s ("+_version.get_versions()['version']+")",
                        help="Get version information")

    add_source_format_arg(parser)
    add_logging_arg_group(parser)

    return parser


def _main(args):
    """
    paramconv main processing

    :param args: argparse arguments
    :return: exit code
    """

    logging.basicConfig(
        level=args.loglevel,
        format='%(asctime)s:%(name)s:%(lineno)s:%(levelname)s:%(message)s'
    )

    # exit if input file does not exist
    if not os.path.isfile(args.input):
        print("Input file not found - {0}".format(args.input))
        exit(8)

    input_filename = args.input
    output_filename = args.output
    if not args.output:
        output_filename = input_filename + ".out"

    # read file to string
    with open(input_filename, 'rb') as infile:
        input_file = infile.read()

    print("{0} bytes read from {1}".format(len(input_file), input_filename))

    # deblock the file
    input_file = unblock(input_file)

    # convert the file
    output_list = [
        _convert(record, args.sourceformat) for record in input_file
    ]

    # re-block the data
    output_data = block(output_list)

    # save to file
    with open(output_filename, "wb") as output_file:
        output_file.write(output_data)

    print("{0} bytes written to {1}".format(len(output_data), output_filename))
    print("{0} records".format(len(output_list)))

    if args.loglevel == logging.DEBUG:
        print("DEBUG:Input first 5000 bytes")
        hexdump(input_file[:5000])
        print("DEBUG:Output first 5000 bytes")
        hexdump(output_data[:5000])

        print("DEBUG:Input last 5000 bytes")
        hexdump(input_file[len(output_data)-5000:len(input_file)])
        print("DEBUG:Output last 5000 bytes")
        hexdump(output_data[len(output_data)-5000:len(output_data)])

    print("Done!")


def _convert(record, source_format):
    """
    Convert record between encoding scheme

    :param record: data to be converted
    :param source_format: ebcdic or ascii
    :return: converted record
    """
    # convert data
    if source_format == "ebcdic":
        record = _convert_text_eb2asc(record)
    else:
        record = _convert_text_asc2eb(record)
    return record


if __name__ == "__main__":
    _main(_get_cli_parser().parse_args())



from __future__ import absolute_import
import os.path
import sys
import hexdump

from unittest import TestCase

from mciutil.mciutil import block, _convert_text_asc2eb, b
from mciutil.cli.mideu import _get_cli_parser, _main
from mciutil.cli.common import get_config_filename, filter_dictionary

TEST_ASCII_IPM_FILENAME = "build/test/test_ascii_ipm.in"
TEST_EBCDIC_IPM_FILENAME = "build/test/test_ebcdic_ipm.in"
HEADER_LINE = (
    "MTI,DE2,DE3,DE4,DE12,DE14,DE22,DE23,DE24,DE25,DE26,DE30,DE31,DE33,"
    "DE37,DE38,DE40,DE41,DE42,DE48,DE49,DE50,DE63,DE71,DE73,DE93,DE94,"
    "DE95,DE100,PDS0023,PDS0052,PDS0122,PDS0148,PDS0158,PDS0165,DE43_NAME,"
    "DE43_SUBURB,DE43_POSTCODE,ICC_DATA\n"
)

DETAIL_LINE = (
    "1144,444455*******555,111111,000000009999,201508151715,,123456789012,"
    ",333,,1234,,57995799120000001230612,123456,,123456,,,579942111111111,"
    "0001001Y,999,,0000000000000001,12345678,,,999999,,,,,,,,,BIG BOBS,"
    "ANNERLEY,4103,\n"
)


class CommandLineTestCase(TestCase):
    """
    Base TestCase class, sets up a CLI parser
    """
    def setUp(self):
        self.parser = _get_cli_parser()
        create_test_ascii_ipm_file()
        create_test_ebcdic_ipm_file()
        create_test_ascii_de55_ipm_file()


class MideuCommonTestCase(CommandLineTestCase):
    def test_enter_with_no_subcommand(self):
        """
        try calling standard entry without sub command.. should return help text
        :return:
        """
        # run with no parms.. argparse should fail
        self.assertRaises(SystemExit, lambda: self.parser.parse_args())
        # run with no parms.. pass none
        _main(None)


class MideuExtractTestCase(CommandLineTestCase):

    def test_with_empty_args(self):
        # with self.assertRaises(SystemExit):
        #    self.parser.parse_args(["extract"])
        self.assertRaises(SystemExit,
                          lambda: self.parser.parse_args(["extract"]))

    def test_with_bad_filename(self):
        test_file = 'abc.txt'
        args = self.parser.parse_args(["extract", test_file])
        self.assertRaises(SystemExit, lambda: _main(args))
        # with self.assertRaises(SystemExit) as stat:
        #     _main(args)
        # self.assertEqual(stat.exception.code, 8)

    def test_with_ascii_file_only(self):
        args = self.parser.parse_args(["extract", "-s", "ascii",
                                       TEST_ASCII_IPM_FILENAME])
        _main(args)
        with open(TEST_ASCII_IPM_FILENAME + ".csv", 'r') as csv_file:
            csv_file_lines = csv_file.readlines()
            print(csv_file_lines)

        # check there are the right number of lines
        # Py26 does not support getheader so will have one less line
        if sys.version_info[0] == 2 and sys.version_info[1] == 6:
            self.assertEqual(len(csv_file_lines), 5)
        else:
            self.assertEqual(len(csv_file_lines), 6)

        # same issue here - 2.6 has no header
        start_line = 1
        if sys.version_info[0] == 2 and sys.version_info[1] == 6:
            start_line = 0
        else:
            self.assertEqual(csv_file_lines[0], HEADER_LINE)

        for line in range(start_line, start_line+5):
            self.assertEqual(csv_file_lines[line], DETAIL_LINE)

    def test_with_ebcdic_file_only(self):
        args = self.parser.parse_args(["extract", "-d",
                                       TEST_EBCDIC_IPM_FILENAME])
        _main(args)
        with open(TEST_EBCDIC_IPM_FILENAME + ".csv", 'r') as csv_file:
            csv_file_lines = csv_file.readlines()
            print(csv_file_lines)

        # check there are the right number of lines
        # Py26 does not support getheader so will have one less line
        if sys.version_info[0] == 2 and sys.version_info[1] == 6:
            self.assertEqual(len(csv_file_lines), 5)
        else:
            self.assertEqual(len(csv_file_lines), 6)

        # same issue here - 2.6 has no header
        start_line = 1
        if sys.version_info[0] == 2 and sys.version_info[1] == 6:
            start_line = 0
        else:
            self.assertEqual(csv_file_lines[0], HEADER_LINE)

        for line in range(start_line, start_line+5):
            self.assertEqual(csv_file_lines[line], DETAIL_LINE)

    def test_with_ascii_de55_values(self):
        args = self.parser.parse_args(["extract", "-s", "ascii",
                                       "build/test/test_ascii_de55_ipm.in"])
        _main(args)


class CsvOutputTest(TestCase):
    def test_output_byte_field(self):
        field = b"this is some text"
        self.assertEqual(field.decode(), 'this is some text')


class MongoOutputTest(TestCase):
    def test_mongo_output(self):
        pass


class FilteredDictionaryTest(TestCase):
    def test_filter_dict(self):
        dict = {"a": b("123"), "b": b("456"), "c": b("789")}
        field_list = ["a", "c"]
        expected_dict = {"a": "123", "c": "789"}
        actual_dict = filter_dictionary(dict, field_list)
        self.assertEqual(len(actual_dict), 2)
        self.assertEqual(actual_dict, expected_dict)


class GetConfigFilenameTest(TestCase):
    def test_module_default(self):
        filename = get_config_filename("test.conf")
        print(filename)


class MideuConvertTestCases(CommandLineTestCase):
    def test_with_ascii_file_only(self):
        print("************ ASCII IN ****************")
        with open(TEST_ASCII_IPM_FILENAME, "rb") as ascii_file:
            ascii_data = ascii_file.read()
            hexdump.hexdump(ascii_data)

        args = self.parser.parse_args(["convert", "-s", "ascii",
                                       TEST_ASCII_IPM_FILENAME])
        _main(args)

        # Convert file with DE55 in it
        args = self.parser.parse_args(["convert", "-s", "ascii",
                                       "build/test/test_ascii_de55_ipm.in"])
        _main(args)

        print("************ EBCDIC IN ****************")
        with open(TEST_ASCII_IPM_FILENAME + ".out", "rb") as ebcdic_file:
            ebcdic_data = ebcdic_file.read()
            hexdump.hexdump(ebcdic_data)

        args = self.parser.parse_args(["convert",
                                       TEST_ASCII_IPM_FILENAME + ".out"])
        _main(args)

        with open(TEST_ASCII_IPM_FILENAME + ".out.out", "rb") as ascii2_file:
            ascii2_data = ascii2_file.read()

        self.assertEqual(ascii2_data, ascii_data)


def create_test_ebcdic_ipm_file():
    message_raw = _convert_text_asc2eb(b("1144")) + \
        b("\xF0\x10\x05\x42\x84\x61\x80\x02\x02\x00\x00\x04"
          "\x00\x00\x00\x00") + \
        _convert_text_asc2eb(b(
            "164444555544445555111111000000009999201508"
            "151715123456789012333123423579957991200000"
            "012306120612345612345657994211111111145BIG"
            " BOBS\\70 FERNDALE ST\\ANNERLEY\\4103  QLD"
            "AUS0080001001Y9990160000000000000001123456"
            "7806999999")
        )
    # add 5 records to a list
    message_list = [message_raw for x in range(5)]
    block_and_write_list(message_list, TEST_EBCDIC_IPM_FILENAME)


def create_test_ascii_ipm_file():
    """
    Create a test IPM file to perform cli testing with
    """
    message_raw = b(
        "1144\xF0\x10\x05\x42\x84\x61\x80\x02\x02\x00\x00\x04"
        "\x00\x00\x00\x00"
        "1644445555444455551111110000000099992015081517151234"
        "5678901233312342357995799120000001230612061234561234"
        "5657994211111111145BIG BOBS\\70 FERNDALE ST\\ANNERLE"
        "Y\\4103  QLDAUS0080001001Y99901600000000000000011234"
        "567806999999"
    )
    # add 5 records to a list
    message_list = [message_raw for x in range(5)]
    block_and_write_list(message_list, TEST_ASCII_IPM_FILENAME)


def block_and_write_list(message_list, file_name):
    # block and write to file
    if not os.path.exists(os.path.dirname(file_name)):
        os.makedirs(os.path.dirname(file_name))
    with open(file_name, 'wb') as output_file:
        output_file.write(block(message_list))


def create_test_ascii_de55_ipm_file():
    message_raw = b(
        "1240" + "\xf0"
        "\x10\x07\xc2\x8d\xe1\x82\x02\x02\x00\x00\x04\x00\x00\x00\x00" + "1"
        "65204230010000012000000000000063"
        "20015102901120601010109014C00120"
        "01401599423752309053030010113511"
        "09110000000988503314801120609054"
        "9120000000290000010364     65MER"
        "CH TERMINAL SETT1\\152 Edward Str"
        "eetq\\BRISBANE\\4000      QLDAUS04"
        "80023003NA 014800403620158012   "
        "       750165001M036052"
        "\x9f\x26\x08\x3e\x24\x24\xed\xa3\x69"
        "\xaa\x47\x9f\x36\x02\x04\xdd\x82\x02\x20\x00\x9f\x02\x06\x00\x00"
        "\x00\x00\x14\x50\x9f\x03\x06\x00\x00\x00\x00\x00\x00\x9f\x27\x01"
        "\x80\x9f\x34\x03\x1f\x00\x00\x9f\x53\x01\xb5"
        "016 M"
        "DP388S6Q1015  000000921100000009"
        "885"
    )

    filename = "build/test/test_ascii_de55_ipm.in"

    message_list = [message_raw for _ in range(5)]
    block_and_write_list(message_list, filename)



"""
mciutil.cli.common

provides common functions for cli tools
"""

import os
import sys
import logging
import csv

LOGGER = logging.getLogger(__name__)


def update_cli_progress(current_val, end_val, bar_length=20):
    """
    Provides cli based progress meter

    :param current_val: the current counter
    :param end_val: the total
    :param bar_length: How long is the progress bar
    :return: None
    """
    percent = float(current_val) / end_val
    hashes = '#' * int(round(percent * bar_length))
    spaces = ' ' * (bar_length - len(hashes))
    sys.stdout.write(
        "\rProcessing: [{0}] {1}%".format(
            hashes + spaces, int(round(percent * 100))))
    sys.stdout.flush()


def add_source_format_arg(parser):
    """
    Adds source format option to parser

    :param parser: the parser to add the source format option to
    :return: None
    """
    parser.add_argument(
        "-s", "--sourceformat",
        help="encoding format of source file",
        choices=["ebcdic", "ascii"],
        default="ebcdic"
    )


def add_logging_arg_group(parser):
    """
    Adds logging options to parser
    :param parser: the argparse parser to add logging options to
    :return: None
    """
    logging_arg_group = parser.add_argument_group("logging options")
    logging_arg_group.add_argument("-d", "--debug",
                                   help="turn debugging output on",
                                   action="store_const",
                                   dest="loglevel",
                                   const=logging.DEBUG,
                                   default=logging.WARNING)
    logging_arg_group.add_argument("-v", "--verbose",
                                   help="turn information output on",
                                   action="store_const",
                                   dest="loglevel",
                                   const=logging.INFO)


def get_config_filename(config_filename):
    """
    Determines the config file to use.

    Starts with current directory, then user home directory and
    finally module installed default config

    :param config_filename: the config filename to use
    :return: the full path and filename of config
    """
    current_dir = os.getcwd()
    user_home_dir = os.path.expanduser("~")

    if os.path.isfile(current_dir + "/" + config_filename):
        return current_dir + "/" + config_filename
    elif os.path.isfile(user_home_dir + "/." + config_filename):
        return user_home_dir + "/." + config_filename
    else:
        module_dir = os.path.dirname(os.path.abspath(__file__))
        return module_dir + "/" + config_filename


def add_to_csv(data_list, field_list, output_filename):
    """
    Writes data to CSV file

    :param data_list: list of dictionaries that contain the data to be loaded
    :param field_list: list of fields in the dictionary to be loaded
    :param output_filename: filename for output CSV file
    :return: None
    """
    filtered_data_list = filter_data_list(data_list, field_list)

    with open(output_filename, "w") as output_file:
        writer = csv.DictWriter(output_file,
                                fieldnames=field_list,
                                extrasaction="ignore",
                                lineterminator="\n")
        # python 2.6 does not support writeheader() so skip
        if sys.version_info[0] == 2 and sys.version_info[1] == 6:
            pass
        else:
            writer.writeheader()

        writer.writerows(filtered_data_list)
    LOGGER.info("%s records written", len(data_list))


def filter_data_list(data_list, field_list):
    """
    Takes list of dictionaries and returns new list filtered to only
    return the field keys provided in field_list and the values
    decoded to unicode.

    :param data_list: the list of dictionaries
    :param field_list: the list of string keys to return
    :return: filtered data list
    """
    return [filter_dictionary(item, field_list) for item in data_list]


def filter_dictionary(dictionary, field_list):
    """
    Takes dictionary and list of elements and returns dictionary with just
    elements specified. Also decodes the items to unicode

    :param dictionary: the dictionary to filter
    :param field_list: list containing keys to keep
    :return: dictionary with just keys from list. All values decoded
    """
    return_dictionary = {}
    for item in dictionary:
        if item in field_list:
            return_dictionary[item] = dictionary[item].decode()

    return return_dictionary


# The following code works in py2.7 and up.. just not 2.6
#     def filter_dictionary(dictionary, field_list):
#         """
#         Takes dictionary and list of elements and returns dictionary with
#         just elements specified. Also decodes the items to unicode
#         """
#         print("{},{}".format(sys.version_info.major, sys.version_info.minor))
#         return {item: dictionary[item].decode() for item in dictionary
#                 if item in field_list}



# -*- coding: utf-8 -*-
"""
mciutil
=======
Shared functions for working with MasterCard files
"""
from .mciutil import (
    block, unblock, vbs_pack, vbs_unpack, get_message_elements,
    flip_message_encoding, b,
)

__author__ = 'Anthony Delosa'
__email__ = 'adelosa@gmail.com'

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions



"""
mciutil.cli.extract

provides functionality for mideu sub-command extract
"""

from __future__ import print_function

import logging
import yaml
import pymongo

from mciutil import unblock, get_message_elements
from mciutil.cli.common import (
    get_config_filename,
    add_to_csv,
    filter_data_list,
)

LOGGER = logging.getLogger(__name__)


def extract_command(args):
    """
    extract command
    :param args: arg object
    :return:
    """

    # Read input file
    with open(args.input, 'rb') as infile:
        input_file = infile.read()
    LOGGER.info("%s bytes read from %s", len(input_file), args.input)

    # Unblock input
    input_file = unblock(input_file)

    # get config filename
    config_filename = get_config_filename("mideu.yml")
    LOGGER.info("Config file: %s", config_filename)

    # load the config from yaml file
    with open(config_filename, 'r') as config_file:
        config = yaml.load(config_file)

    # parse the records
    output_list = [
        get_message_elements(
            record,
            config["bit_config"],
            args.sourceformat
        ) for record in input_file
    ]

    print("\nCompleted processing {0} records".format(len(input_file)))

    # write to output
    if args.mongo or args.mongohost or args.mongodb:
        mongo_config = config["mongo_config"]
        if args.mongohost:
            mongo_config["host"] = args.mongohost
        if args.mongodb:
            mongo_config["db"] = args.mongodb

        add_to_mongo(
            output_list,
            config['output_data_elements'],
            mongo_config)

    else:  # default write to csv
        if args.csvoutputfile:
            csv_output_filename = args.csvoutputfile
        else:
            csv_output_filename = args.input + ".csv"
        add_to_csv(
            output_list,
            config['output_data_elements'],
            csv_output_filename
        )


def add_to_mongo(data_list, field_list, mongo_config):
    """
    Loads data into a mongo db collection

    :param data_list: list of dictionaries that contain the data to be loaded
    :param field_list: list of fields in the dictionary to be loaded
    :param mongo_config: config dictionary required to connect to mongo
    :return: result from mongo insert
    """
    LOGGER.info("Connecting to mongo at %s", mongo_config["host"])
    client = pymongo.MongoClient("mongodb://" + mongo_config["host"])
    db_client = client[mongo_config["db"]]

    LOGGER.info("Deleting existing items")
    db_client.mastercardtransactions.delete_many({})

    LOGGER.info("Filtering %s items", len(data_list))
    filtered_data_list = filter_data_list(data_list, field_list)

    LOGGER.info("Loading %s items", len(filtered_data_list))
    return db_client.mastercardtransactions.insert_many(filtered_data_list)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import versioneer

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
    'PyYAML', 'hexdump', 'bitarray', 'pymongo', 'argparse'
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='mciutil',
    description='MasterCard file utilities',
    long_description=readme + '\n\n' + history,
    author='Anthony Delosa',
    author_email='adelosa@gmail.com',
    url='https://github.com/adelosa/mciutil',
    packages=[
        'mciutil', 'mciutil.cli'
    ],
    package_dir={'mciutil':
                 'mciutil'},
    include_package_data=True,
    install_requires=requirements,
    entry_points={
        'console_scripts': [
            'mideu = mciutil.cli.mideu:cli_entry',
            'paramconv = mciutil.cli.paramconv:cli_entry',
        ]
    },
    license='BSD',
    zip_safe=False,
    keywords='mciutil',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
)



"""
mciutil.cli.convert

provides functionality for mideu subcommand convert
"""

from __future__ import print_function

import logging
import yaml

from mciutil import flip_message_encoding, unblock, block
from mciutil.cli.common import get_config_filename

LOGGER = logging.getLogger(__name__)


def convert_command(args):
    """
    convert command
    :param args: arg object
    :return:
    """

    # Read input file
    with open(args.input, 'rb') as input_file:
        input_data = input_file.read()
    LOGGER.info("%s bytes read from %s", len(input_data), args.input)

    # Unblock input
    input_data = unblock(input_data)

    # get config filename
    config_filename = get_config_filename("mideu.yml")
    LOGGER.info("Config file: %s", config_filename)

    # load the config from yaml file
    with open(config_filename, 'r') as config_file:
        config = yaml.load(config_file)

    output_records = [
        flip_message_encoding(
            record,
            config["bit_config"],
            args.sourceformat
        ) for record in input_data
    ]

    output_data = block(output_records)

    print("\nCompleted processing {0} records".format(len(input_data)))

    # save to file
    output_filename = args.input + ".out"
    with open(output_filename, "wb") as output_file:
        output_file.write(output_data)



from __future__ import absolute_import
from unittest import TestCase
import hexdump

# Public import
import mciutil

# Private functions
from mciutil.mciutil import (
    _convert_text_asc2eb, _get_de43_fields, _mask_pan, b, flip_message_encoding, _get_icc_fields
)

CONFIG = {
    'data_elements':
        {1: {'field_type': 'FIXED',
             'field_name': 'Bitmap secondary', 'field_length': 8},
         2: {'field_type': 'LLVAR',
             'field_name': 'PAN', 'field_processor': 'PAN', 'field_length': 0},
         3: {'field_type': 'FIXED',
             'field_name': 'Processing code', 'field_length': 6},
         4: {'field_type': 'FIXED',
             'field_name': 'Amount transaction', 'field_length': 12},
         71: {'field_type': 'FIXED',
              'field_name': 'Message number', 'field_length': 8},
         73: {'field_type': 'FIXED',
              'field_name': 'Date, Action', 'field_length': 6},
         12: {'field_type': 'FIXED',
              'field_name': 'Date/Time local transaction', 'field_length': 12},
         22: {'field_type': 'FIXED',
              'field_name': 'Point of service data code', 'field_length': 12},
         23: {'field_type': 'FIXED',
              'field_name': 'Card sequence number', 'field_length': 3},
         24: {'field_type': 'FIXED',
              'field_name': 'Function code', 'field_length': 3},
         26: {'field_type': 'FIXED',
              'field_name': 'Card acceptor business code', 'field_length': 4},
         94: {'field_type': 'LLVAR',
              'field_name': 'Transaction originator institution ID',
              'field_length': 0},
         31: {'field_type': 'LLVAR',
              'field_name': 'Acquirer reference data', 'field_length': 23},
         33: {'field_type': 'LLVAR',
              'field_name': 'Forwarding inst id', 'field_length': 0},
         38: {'field_type': 'FIXED',
              'field_name': 'Approval code', 'field_length': 6},
         40: {'field_type': 'FIXED',
              'field_name': 'Service code', 'field_length': 3},
         42: {'field_type': 'FIXED',
              'field_name': 'Card acceptor Id', 'field_length': 15},
         43: {'field_type': 'LLVAR',
              'field_name': 'Card acceptor name/location',
              'field_processor': 'DE43',
              'field_length': 0},
         48: {'field_type': 'LLLVAR',
              'field_name': 'Additional data', 'field_processor': 'PDS',
              'field_length': 0},
         49: {'field_type': 'FIXED',
              'field_name': 'Currency code', 'field_length': 3},
         55: {'field_type': 'LLLVAR',
              'field_name': 'ICC system related data',
              'field_processor': 'ICC',
              'field_length': 255},
         63: {'field_type': 'LLLVAR',
              'field_name': 'Transaction lifecycle Id', 'field_length': 16}},
    'default_output_elements':
        ['MTI', 'DE2', 'DE3', 'DE4', 'DE12', 'DE22', 'DE24', 'DE26', 'DE31',
         'DE33', 'DE38', 'DE42', 'DE49', 'DE71', 'DE94', 'DE73', 'DE23',
         'DE40', 'DE63', 'PDS0023', 'PDS0052', 'PDS0148', 'PDS0158', 'PDS0165',
         'DE43_NAME', 'DE43_SUBURB', 'DE43_POSTCODE']
    }


class TestGetMessageElements(TestCase):

    def test_get_message_elements_ascii(self):
        message_raw = b(
                      "1144\xF0\x10\x05\x42\x84\x61\x80\x02\x02\x00\x00\x04"
                      "\x00\x00\x00\x00" +
                      "1644445555444455551111110000000099992015081517151234"
                      "5678901233312342357995799120000001230612061234561234"
                      "5657994211111111145BIG BOBS\\70 FERNDALE ST\\ANNERLE"
                      "Y\\4103  QLDAUS0080001001Y99901600000000000000011234"
                      "567806999999")

        message_elements = mciutil.get_message_elements(
            message_raw, CONFIG['data_elements'], 'ascii')

        print(message_elements)

        # print message_elements
        self.assertEqual(message_elements["DE2"], b"444455*******555")
        self.assertEqual(message_elements["DE3"], b"111111")
        self.assertEqual(message_elements["DE4"], b"000000009999")
        self.assertEqual(message_elements["PDS0001"], b"Y")

    def test_get_message_elements_ebcdic(self):
        message_raw = (
            _convert_text_asc2eb(b("1144")) +
            b("\xF0\x10\x05\x42\x84\x61\x80\x02\x02\x00\x00\x04") +
            b("\x00\x00\x00\x00") +
            _convert_text_asc2eb(b("164444555544445555111111000000009999201508"
                                   "151715123456789012333123423579957991200000"
                                   "012306120612345612345657994211111111145BIG"
                                   " BOBS\\70 FERNDALE ST\\ANNERLEY\\4103  QLD"
                                   "AUS0080001001Y9990160000000000000001123456"
                                   "7806999999"))
        )
        message_elements = mciutil.get_message_elements(
            message_raw, CONFIG['data_elements'], 'ebcdic')

        # print message_elements
        self.assertEqual(message_elements["DE2"], b"444455*******555")
        self.assertEqual(message_elements["DE3"], b"111111")
        self.assertEqual(message_elements["DE4"], b"000000009999")

    def test_get_de43_elements(self):
        de43_raw = b("AAMI                  \\36 WICKHAM TERRACE             "
                     "              \\BRISBANE     \\4000      QLDAUS")
        de43_elements = _get_de43_fields(de43_raw)
        self.assertEqual(de43_elements["DE43_NAME"], b"AAMI")
        self.assertEqual(de43_elements["DE43_ADDRESS"], b"36 WICKHAM TERRACE")
        self.assertEqual(de43_elements["DE43_SUBURB"], b"BRISBANE")
        self.assertEqual(de43_elements["DE43_POSTCODE"], b"4000")
        self.assertEqual(de43_elements["DE43_STATE"], b"QLD")
        self.assertEqual(de43_elements["DE43_COUNTRY"], b"AUS")

    def test_vbs_to_line(self):
        vbs_record = b("\x00\x00\x00\x0A1234567890\x00\x00\x00\x0A1234567890")
        records = mciutil.vbs_unpack(vbs_record)
        print("Length of output =", len(records))
        print(records)

    def test_line_to_vbs(self):
        linebreakdata = [b('1234567890'), b('1234567890')]
        vbsdata = mciutil.vbs_pack(linebreakdata)
        hexdump.hexdump(vbsdata)
        self.assertEqual(vbsdata,
                         b"\x00\x00\x00\x0A1234567890\x00\x00\x00\x0A123456"
                         b"7890\x00\x00\x00\x00",
                         )

    def test_unblock(self):
        umodedata = ((b("\x00\x00\x00\x0A1234567890") * 72) +
                     b("\x00\x00\x00\x0A\x40\x401234567890"))
        records = mciutil.unblock(umodedata)
        print(len(records))
        print(records[0])
        print(records[72])

    def test_block(self):
        linebreakdata = []
        for x in range(0, 73):
            linebreakdata.append(b("1234567890"))
        output = mciutil.block(linebreakdata)
        hexdump.hexdump(output)
        input = mciutil.unblock(output)
        print(len(input))
        print(input)

    def test_mask_pan(self):
        card_number = b("1234567890123456")
        self.assertEqual(_mask_pan(card_number), b"123456*******456")

    def test_flip_message_elements_ascii(self):
        message_raw = b(
          "1144\xF0\x10\x05\x42\x84\x61\x80\x02\x02\x00\x00\x04"
          "\x00\x00\x00\x00" +
          "1644445555444455551111110000000099992015081517151234"
          "5678901233312342357995799120000001230612061234561234"
          "5657994211111111145BIG BOBS\\70 FERNDALE ST\\ANNERLE"
          "Y\\4103  QLDAUS0080001001Y99901600000000000000011234"
          "567806999999"
        )

        message_elements = flip_message_encoding(
            message_raw, CONFIG['data_elements'], 'ascii')
        print("************* E B C D I C ******************")
        hexdump.hexdump(message_elements)
        message_elements = flip_message_encoding(
            message_elements, CONFIG['data_elements'], 'ebcdic')
        print("************* A S C I I ********************")
        hexdump.hexdump(message_elements)
        print("********************************************")
        self.assertEqual(message_raw, message_elements)


class TestDe55unblock(TestCase):

    de55_field = b(
        "\x9f\x26\x08\x3e\x24\x24\xed\xa3\x69"
        "\xaa\x47\x9f\x36\x02\x04\xdd\x82\x02\x20\x00\x9f\x02\x06\x00\x00"
        "\x00\x00\x14\x50\x9f\x03\x06\x00\x00\x00\x00\x00\x00\x9f\x27\x01"
        "\x80\x9f\x34\x03\x1f\x00\x00\x9f\x53\x01\xb5"
    )

    def test_get_de55_fields(self):
        return_dict = _get_icc_fields(self.de55_field)
        print(return_dict)
        self.assertNotEqual(return_dict.get("TAG9F26", False), False)
        self.assertEqual(return_dict['TAG9F26'], b("3e2424eda369aa47"))
        self.assertNotEqual(return_dict.get("TAG9F36", False), False)
        self.assertEqual(return_dict['TAG9F36'], b("04dd"))
        self.assertNotEqual(return_dict.get("TAG82", False), False)
        self.assertEqual(return_dict['TAG82'], b("2000"))



from __future__ import absolute_import
import os
from unittest import TestCase
from mciutil import b, block
from mciutil.cli.paramconv import _get_cli_parser, _main

TEST_ASCII_MPE_PARAM_FILENAME = "build/test/test_ascii_mpe.in"


class CommandLineTestCase(TestCase):
    """
    Base TestCase class, sets up a CLI parser
    """
    def setUp(self):
        self.parser = _get_cli_parser()
        write_test_file(create_test_mpe_parameter_extract(),
                        TEST_ASCII_MPE_PARAM_FILENAME)


class ParamConvTestCase(CommandLineTestCase):

    def test_with_empty_args(self):
        # with self.assertRaises(SystemExit):
        #     self.parser.parse_args([])
        self.assertRaises(SystemExit,
                          lambda: self.parser.parse_args([]))

    def test_with_bad_filename(self):
        test_file = 'abc.txt'
        args = self.parser.parse_args([test_file])
        # with self.assertRaises(SystemExit) as stat:
        #     _main(args)
        # self.assertEquals(stat.exception.code, 8)
        self.assertRaises(SystemExit,
                          lambda: _main(args))

    def test_with_ascii_file(self):
        # convert to ebcdic
        args = self.parser.parse_args(["-s", "ascii",
                                       TEST_ASCII_MPE_PARAM_FILENAME])
        _main(args)
        # convert back to ascii
        args = self.parser.parse_args([TEST_ASCII_MPE_PARAM_FILENAME + ".out"])
        _main(args)

        # check that double converted file == original file
        with open(TEST_ASCII_MPE_PARAM_FILENAME + ".out.out", 'rb') as outfile:
            output_data = outfile.read()

        self.assertEqual(output_data, create_test_mpe_parameter_extract())


def create_test_mpe_parameter_extract():
    """
    Create a test extract file to perform cli testing with
    Contents do no matter..
    """
    message_raw = b("12345678901ABCDEFGHIJKLMNOPQRSTUVWXYZ")
    # add 5 records to a list
    message_list = [message_raw for x in range(5)]
    return block(message_list)

    #  block_and_write_list(message_list, TEST_ASCII_MPE_PARAM_FILENAME)


def write_test_file(message, file_name):
    # block and write to file
    if not os.path.exists(os.path.dirname(file_name)):
        os.makedirs(os.path.dirname(file_name))
    with open(file_name, 'wb') as output_file:
        output_file.write(message)
